package org.xbib.interlibrary.db.medea.postgresql.util;

import org.xbib.datastructures.json.tiny.Json;
import org.xbib.interlibrary.http.HttpHelper;
import org.xbib.interlibrary.action.avail.AvailRequest;
import org.xbib.interlibrary.action.avail.AvailResponse;
import org.xbib.settings.Settings;

import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.util.Map;
import java.util.Objects;
import java.util.logging.Level;
import java.util.logging.Logger;

public class HttpAvail {
    private static final Logger logger = Logger.getLogger(HttpAvail.class.getName());

    private final HttpHelper httpHelper;

    private final String availUrl;

    public HttpAvail(Settings settings,
                     HttpHelper httpHelper) {
        this.httpHelper = httpHelper;
        if (!settings.containsSetting("database.url.avail")) {
            throw new IllegalArgumentException("no setting database.url.avail found");
        }
        this.availUrl = Objects.requireNonNull(settings.get("database.url.avail"), "database.url.avail not defined in settings");
    }

    public AvailResponse avail(AvailRequest availRequest) {
        throw new UnsupportedOperationException();
    }

    public Map<String, Object> avail(Map<String, Object> map) throws IOException {
        String json = availAsJson(availUrl, map);
        return json.isEmpty() ? Map.of() : Json.toMap(json);
    }

    private String availAsJson(String url, Map<String, Object> map) throws IOException {
        StringBuilder sb = new StringBuilder();
        httpHelper.get(url,
                params -> {
                    if (map != null) {
                        params.putAll(map);
                    }
                },
                headers -> {
                    headers.add("accept","application/json");
                },
                response -> {
                    int status = response.getStatus().code();
                    if (status == 200) {
                        sb.append(response.getBodyAsChars(StandardCharsets.UTF_8).toString());
                    } else {
                        logger.log(Level.SEVERE, "HTTP response status = " + status);
                    }
                },
                exception -> logger.log(Level.SEVERE, exception.getMessage(), exception),
                timeout -> logger.log(Level.WARNING, "timeout while waiting for response for " + url));
        return sb.toString();
    }

}
